require 'aws-sdk'

module DashOverlord
  module Models
    module V1
      module DynamoDb
        class Base < OpenStruct
          def self.client
            @client ||= Aws::DynamoDB::Client.new
          end

          def self.create_all(items)
            items.each_slice(25) do |batch|
              client.batch_write_item \
                create_all_params(batch)
            end
          end

          def self.create(attributes = {})
            client.put_item \
              create_params(attributes)
          end

          def self.find_by(attribute)
            result = @client.get_item({
              table_name: table_name,
              key: attribute
            })

            return nil unless result.item

            self.new(result.item)
          end

          def self.all
            DynamoDb::Scan.new \
              client: client,
              table_name: table_name,
              resource_class: self
          end

          def result(args)
            self
          end

          # def self.page(page, last_evaluated_key)
          #   DynamoDb::Scan.new \
          #     current_page: page,
          #     last_evaluated_key: last_evaluated_key,
          #     client: client,
          #     table_name: table_name,
          #     resource_class: self
          # end

          def self.ransack(filters)
            return unless filters

            DynamoDb::Scan.new \
              client: client,
              table_name: table_name,
              resource_class: self,
              filters: filters
          end

          def self.table_name
            return @table_name if defined?(@table_name)

            @table_name = name.split('::').last.underscore
          end

          alias attributes marshal_dump

          def assign_attributes(attributes)
            attributes.each { |key, value| send("#{key}=", value) }
          end

          private

          def self.create_params(attributes)
            {
              table_name: table_name,
              item: attributes
            }
          end

          def self.create_all_params(items)
            {
              request_items: {
                "#{table_name}" => items_params(items)
              }
            }
          end

          def self.items_params(items)
            items.map do |item|
              {
                put_request: {
                  item: item
                }
              }
            end
          end

          def self.scan_params
            {
              table_name: table_name,
              exclusive_start_key: @last_evaluated_key,
              limit: @per_page
            }
          end

        end
      end
    end
  end
end
