module DashOverlord
  module Models
    module V1
      module DynamoDb
        class Scan < DynamoDb::Base
          # resource.page(page).per(per_page)
          def ransack(filters)
            self.filters = filters

            self
          end

          # def page(page, last_evaluated_key)
          #   self.current_page = page
          #   self.last_evaluated_key = last_evaluated_key

          #   self
          # end

          # def per(per_page)
          #   self.limit = per_page

          #   self
          # end

          def result(distinct)
            self
          end

          def load
            scan_all
          end

          protected ##################### PROTECTED ###########################

          def scan_all
            scan_params = params
            items = []

            loop do
              result = client.query(scan_params)
              items += result.items

              break unless result.last_evaluated_key

              scan_params[:exclusive_start_key] = result.last_evaluated_key
            end

            items.map{ |item| resource_class.new(item) }
          end

          def paginated_scan
            (limit && current_page)
          end

          def scan_page
            result = client.query(params)

            DynamoDb::PaginatedResult.new \
              items: result.items.map{ |item| resource_class.new(item) },
              last_evaluated_key: result.last_evaluated_key,
              exclusive_start_key: last_evaluated_key,
              limit: limit
          end

          def params
            {
              table_name: "#{table_name}",
              exclusive_start_key: last_evaluated_key,
              limit: limit
            }.merge(filter_params)
          end

          def filter_params
            return {} unless filters

            initial_params = {
              key_condition_expression: "#dashboard_id = :dashboard_id",
              filter_expression: [],
              expression_attribute_names: {
                "#dashboard_id" => "dashboard_id"
              },
              expression_attribute_values: {
                ':dashboard_id' => 1
              }
            }

            filters_mapped = \
              filters.each_with_object(initial_params) do |(key, value), object|
                process_filter(key, value, object)

                object
              end

            filters_mapped[:filter_expression] = \
              filters_mapped[:filter_expression].join(' and ')

            filters_mapped
          end

          private

          def process_filter(attribute_expression, value, object)
            attribute = attribute_expression.to_s.gsub('_eq', '')

            object[:expression_attribute_names]["##{attribute}"] = \
              attribute

            object[:expression_attribute_values][":#{attribute}"] = \
              value

            object[:filter_expression] << \
              "##{attribute} = :#{attribute}"
          end
        end
      end
    end
  end
end
