module DashOverlord
  module Serializers

    class Base
      RAILS_HTTP_STATUS = {
        continue: 100,
        switching_protocols: 101,
        processing: 102,
        ok: 200,
        created: 201,
        accepted: 202,
        non_authoritative_information: 203,
        no_content: 204,
        reset_content: 205,
        partial_content: 206,
        multi_status: 207,
        im_used: 226,
        multiple_choices: 300,
        moved_permanently: 301,
        found: 302,
        see_other: 303,
        not_modified: 304,
        use_proxy: 305,
        temporary_redirect: 307,
        bad_request: 400,
        unauthorized: 401,
        payment_required: 402,
        forbidden: 403,
        not_found: 404,
        method_not_allowed: 405,
        not_acceptable: 406,
        proxy_authentication_required: 407,
        request_timeout: 408,
        conflict: 409,
        gone: 410,
        length_required: 411,
        precondition_failed: 412,
        request_entity_too_large: 413,
        request_uri_too_long: 414,
        unsupported_media_type: 415,
        requested_range_not_satisfiable: 416,
        expectation_failed: 417,
        unprocessable_entity: 422,
        locked: 423,
        failed_dependency: 424,
        upgrade_required: 426,
        internal_server_error: 500,
        not_implemented: 501,
        bad_gateway: 502,
        service_unavailable: 503,
        gateway_timeout: 504,
        http_version_not_supported: 505,
        insufficient_storage: 507,
        not_extended: 510
      }.freeze

      def self.to_hash(context, method = nil, options = {})
        object = method ? context.send(method) : context

        hash = (method == false) ? { data: nil } : serialize(object, options)

        hash.merge \
          code: RAILS_HTTP_STATUS[context.status.to_sym],
          status: context.status,
          message: context.error.message
      end

      def self.serialize(object, options)
        data = self.new(object, options).serialize

        (root_name != '') ? { root_name => data } : data
      end

      def self.root(root_name)
        @root_name = root_name
      end

      def self.root_name
        @root_name ||= (superclass.respond_to?(:root_name) \
          ? (superclass.root_name || '')
          : ''
        ).dup
      end

      def self.expose(attribute, config = {})
        attributes << [attribute, { as: attribute }.merge(config)]
      end

      def self.attributes
        @attributes ||= (superclass.respond_to?(:attributes) \
          ? (superclass.attributes || [])
          : []
        ).dup
      end

      def initialize(object, options = {})
        @object, @options = object, options
      end

      attr_reader :object, :options

      def serialize
        object.respond_to?(:map) ? _serialize_array : _serialize_hash
      end

      protected

      def _serialize_array
        object.map do |single_object|
          self.class.new(single_object, options).serialize
        end
      end

      def _serialize_hash
        self.class.attributes.each_with_object({}) do |(method, config), hash|
          next if config[:if] && !send(config[:if])

          data = respond_to?(method) ? send(method) : object.send(method)

          hash[config[:as]] =
            config[:using] ? config[:using].new(data, options).serialize : data
        end
      end
    end

  end
end
