require 'ostruct'
require 'object_attorney'

module DashOverlord
  module UseCases

    class Base
      AbortError = Class.new(StandardError)

      def self.perform(context = nil, attributes = nil)
        instance = self.new(context, attributes)

        begin
          instance.perform
        rescue AbortError
          nil
        end

        instance.final

        instance.context
      end

      def self.context_reader(*methods)
        methods.each { |method| define_method(method) { context.send(method) } }
      end

      def self.target(name, options = {})
        # Gem that allows you to run ActiveRecord validations against a model
        self.include ObjectAttorney

        defend(name, options)

        if defendant_options[:in].present?
          context_reader defendant_options[:in]
        else
          context_reader name
        end
      end

      def initialize(context = nil, attributes = nil)
        @context = context if context.is_a?(Context)

        @context ||= Context.new(context || {})

        @context.assign_attributes(attributes || {})
      end

      attr_reader :context

      def perform; end

      def final; end

      def target_valid?
        respond_to?(:valid?) ? valid? : false
      end

      protected ######################## PROTECTED #############################

      def status
        context.status
      end

      def invoke(use_case, attributes = nil)
        use_case.perform(context, attributes)
      end

      def invoke!(use_case, attributes = nil)
        new_context = invoke(use_case, attributes)

        raise(AbortError) unless new_context.ok?

        new_context
      end

      def failure(status, message = nil)
        context.add_error(status, message, self.class.name)
      end

      def failure!(status, message = nil)
        failure(status, message) && raise(AbortError)
      end
    end

  end
end
