require 'ostruct'

module DashOverlord
  module UseCases

    class Context < OpenStruct
      alias attributes marshal_dump

      if defined?(ActiveModel) && defined?(ActiveModel::Serialization)
        include ActiveModel::Serialization
      end

      def assign_attributes(attributes)
        attributes.each { |key, value| send("#{key}=", value) }
      end

      def errors
        @errors ||= []
      end

      def add_error(status, message, class_name)
        self.status.send("#{status}!")

        @last_known_error = {
          status: status.to_s,
          message: message.to_s,
          class_name: class_name.to_s
        }

        errors.push(@last_known_error)
      end

      def last_known_error
        @last_known_error ||= {}
      end

      def error
        OpenStruct.new(last_known_error)
      end

      def valid?
        errors.empty?
      end

      alias ok? valid?

      class StatusString < String
        def method_missing(method, *args, &block)
          last_char = method[-1]
          method_name = method[0...-1]

          if last_char == '!'
            replace(method_name)
          elsif last_char == '?'
            self == method_name
          else
            super
          end
        end
      end

      def status
        @status ||= StatusString.new('ok')
      end

      def status=(_)
        raise 'status is a reserved keyword which cannot be set'
      end
    end

  end
end
