module DashOverlord
  module UseCases
    module V1
      module ChartConfigs
        module Create

          class ValidateChartConfig < V1::Base
            ALLOWED_FUNCTIONS = Models::V1::ChartConfig::ALLOWED_FUNCTIONS
            ALLOWED_FONT_STYLES = Models::V1::ChartConfig::ALLOWED_FONT_STYLES

            target :chart_config

            validates_presence_of :source

            validates_presence_of :project_id

            validates_inclusion_of :function,
                                   in: ALLOWED_FUNCTIONS,
                                   message: :invalid

            validates_inclusion_of \
              :source,
              in: ->(chart_config) { chart_config.allowed_sources.map(&:id) },
              message: :invalid,
              allow_blank: true

            validates_numericality_of :title_font_size, only_integer: true

            validates_inclusion_of :title_font_style,
                                   in: ALLOWED_FONT_STYLES,
                                   message: :invalid

            validates_numericality_of :function_axis_font_size,
                                      only_integer: true

            validates_inclusion_of :function_axis_font_style,
                                   in: ALLOWED_FONT_STYLES,
                                   message: :invalid

            validates_numericality_of :function_axis_maximum, allow_nil: true

            validates_numericality_of :function_axis_minimum,
                                      only_integer: true

            validates_numericality_of :function_axis_labels_decimal_places,
                                      only_integer: true,
                                      allow_nil: true

            validates_numericality_of :function_axis_labels_font_size,
                                      only_integer: true

            validates_inclusion_of :function_axis_labels_font_style,
                                   in: ALLOWED_FONT_STYLES,
                                   message: :invalid

            validates_numericality_of :source_axis_font_size,
                                      only_integer: true

            validates_inclusion_of :source_axis_font_style,
                                   in: ALLOWED_FONT_STYLES,
                                   message: :invalid

            def perform
              return if target_valid?

              failure! \
                :unprocessable_entity,
                I18n.t('v1.chart_configs.contains_errors')
            end
          end

        end
      end
    end
  end
end
