module DashOverlord
  module UseCases
    module V1
      module Repositories
        module DynamoDb
          module SearchAndPaginate

            class ApplyPagination < V1::Base
              PAGE_DEFAULT = 1
              PER_PAGE_DEFAULT = 10

              context_reader  :page,
                              :last_evaluated_key,
                              :per_page,
                              :resources,
                              :resource

              def perform
                context.page = fix_page(page)

                if page == 0
                  no_pagination
                else
                  apply_pagination
                end
              end

              protected ###################### PROTECTED ###########################

              def no_pagination
                context.resources = resource.all

                context.per_page = 0
              end

              def apply_pagination
                context.per_page = fix_per_page(per_page)

                context.resource = \
                  resource.page(page, last_evaluated_key).per(per_page)
              end

              def fix_page(page)
                page_or_nil = page.class == String ? page.presence : page

                (page_or_nil || PAGE_DEFAULT).to_i
              end

              def fix_per_page(per_page)
                per_page_or_nil = \
                  per_page.class == String ? per_page.presence : per_page

                (per_page_or_nil || PER_PAGE_DEFAULT).to_i
              end

            end
          end
        end
      end
    end
  end
end