module DashOverlord
  module UseCases
    module V1
      module Shared
        module LogIn

          class ValidateUserModel < V1::Base
            target :user_model

            context_reader :existing_user_model, :user_model_class

            validates_presence_of :email, :password

            validate :validate_email

            validate :validate_password

            def validate_email(user_model)
              return if user_model.email.blank?
              return if existing_user_model.present?

              user_model.errors.add(:email, message('bad_credentials'))
              user_model.errors.add(:password, message('bad_credentials'))
            end

            def validate_password(user_model)
              return if existing_user_model.blank?
              return if user_model.password.blank?
              return if existing_user_model.secured_password == user_model.password

              user_model.errors.add(:email, message('bad_credentials'))
              user_model.errors.add(:password, message('bad_credentials'))
            end

            def perform
              return if target_valid?

              failure! :unprocessable_entity, message('contains_errors')
            end

            protected

            def message(key)
              I18n.t("v1.#{user_model_class.model_name.element}s.#{key}")
            end
          end

        end
      end
    end
  end
end
