require 'kaminari'
require 'kaminari/models/array_extension'

module DashOverlord
  module UseCases
    module V1
      module Shared
        module SearchAndPaginate

          class ApplyPagination < V1::Base
            PAGE_DEFAULT = 1
            PER_PAGE_DEFAULT = 10

            context_reader :page, :per_page, :resources

            def perform
              context.page = fix_page(page)
              context.per_page = fix_per_page(per_page)

              page == 0 ? no_pagination : apply_pagination
            end

            protected

            def no_pagination
              context.per_page = 0
              context.total_pages = 0
              context.total_results = resources.count
            end

            def apply_pagination
              context.resources = paginated_resources.page(page).per(per_page)

              context.page = resources.current_page
            end

            def fix_page(page)
              page_or_nil = page.class == String ? page.presence : page

              (page_or_nil || PAGE_DEFAULT).to_i
            end

            def fix_per_page(per_page)
              per_page_or_nil = \
                per_page.class == String ? per_page.presence : per_page

              (per_page_or_nil || PER_PAGE_DEFAULT).to_i
            end

            private

            def paginated_resources
              if resources.respond_to?(:page)
                resources
              else
                Kaminari.paginate_array(resources)
              end
            end
          end

        end
      end
    end
  end
end
