require 'ostruct'

require 'elasticord/client/base'

module Elasticord
  class Base < OpenStruct
    module ClassMethods
      extend Forwardable

      def client(sub_index = nil, sub_type = nil)
        full_type = [type, sub_type].compact.join('_')


        full_index = [index, sub_index].compact.join('_')

        Client::Base.new \
          Elasticord.elastic_search_client, self, full_index, full_type
      end

      def_delegators :client,
                     :get,
                     :none,
                     :create,
                     :search,
                     :delete_all,
                     :refresh_index,
                     :delete_by_query

      def index
        return @index if defined?(@index)

        @index = (superclass.index || '').dup if superclass.respond_to? :index

        @index = @index.presence || Elasticord::Config.get(:index_namespace)
      end

      def index=(new_index)
        @index = new_index
      end

      def type
        return @type if defined?(@type)

        return (@type = '_all') if name.nil? || name == 'Elasticord::Base'

        @type = name.split('::').last.underscore
      end
    end

    extend ClassMethods

    alias attributes marshal_dump

    def assign_attributes(attributes)
      attributes.each { |key, value| send("#{key}=", value) }
    end
  end
end
