require 'ostruct'

require 'elasticord/client/get'
require 'elasticord/client/index'
require 'elasticord/client/create'
require 'elasticord/client/delete'
require 'elasticord/client/search_builder'

module Elasticord
  module Client
    class Base
      def initialize(elastic_search_client, entity, index, type)
        @elastic_search_client, @entity, @index, @type =
          elastic_search_client, entity, index, type
      end

      attr_reader :index, :type

      def create(attributes = {})
        result = Client::Create.one \
          elastic_search_client, index, type, attributes

        return false unless result

        attributes[:id] ||= result['_id']

        entity.new(attributes)
      end

      def get(id)
        result = Client::Get.one(elastic_search_client, index, type, id)

        return false unless result

        entity.new(result.merge({ 'id' => id }))
      end

      def search(body_params = {})
        none.load(body_params)
      end

      def none
        Client::SearchBuilder.new(elastic_search_client, entity, index, type)
      end

      def delete_all
        Client::Delete.all(elastic_search_client, index)
      end

      def refresh_index
        Client::Index.refresh(elastic_search_client, index)
      end

      protected

      attr_reader :elastic_search_client, :entity
    end
  end
end
